@@ -5,7 +5,6 @@ if sys.version_info >= (3, 9):
 else:
     import pathlib3x as pathlib
 
-import re
 from typing import List, Dict, Union, Callable, Tuple
 import inspect
 import logging
@@ -21,7 +20,7 @@ from . import v30
 from . import v31
 from . import log
 from .request import OperationIndex, HTTP_METHODS
-from .errors import ReferenceResolutionError, SpecError
+from .errors import ReferenceResolutionError
 from .loader import Loader, NullLoader
 from .plugin import Plugin, Plugins
 from .base import RootBase, ReferenceBase, SchemaBase
@@ -51,10 +50,15 @@ class OpenAPI:
 
     @classmethod
     def load_sync(
-        cls, url, session_factory: Callable[[], httpx.Client] = httpx.Client, loader=None, plugins: List[Plugin] = None
+        cls,
+        url,
+        session_factory: Callable[[], httpx.Client] = httpx.Client,
+        loader=None,
+        plugins: List[Plugin] = None,
+        use_operation_tags: bool = False,
     ):
         resp = session_factory().get(url)
-        return cls._load_response(url, resp, session_factory, loader, plugins)
+        return cls._load_response(url, resp, session_factory, loader, plugins, use_operation_tags)
 
     @classmethod
     async def load_async(
@@ -63,16 +67,17 @@ class OpenAPI:
         session_factory: Callable[[], httpx.AsyncClient] = httpx.AsyncClient,
         loader=None,
         plugins: List[Plugin] = None,
+        use_operation_tags: bool = False,
     ):
         async with session_factory() as client:
             resp = await client.get(url)
-        return cls._load_response(url, resp, session_factory, loader, plugins)
+        return cls._load_response(url, resp, session_factory, loader, plugins, use_operation_tags)
 
     @classmethod
-    def _load_response(cls, url, resp, session_factory, loader, plugins):
+    def _load_response(cls, url, resp, session_factory, loader, plugins, tags):
         if resp.is_redirect:
             raise ValueError(f'Redirect to {resp.headers.get("Location","")}')
-        return cls.loads(url, resp.text, session_factory, loader, plugins)
+        return cls.loads(url, resp.text, session_factory, loader, plugins, tags)
 
     @classmethod
     def load_file(
@@ -82,12 +87,13 @@ class OpenAPI:
         session_factory: Callable[[], httpx.AsyncClient] = httpx.AsyncClient,
         loader: Loader = None,
         plugins: List[Plugin] = None,
+        use_operation_tags: bool = False,
     ):
         assert loader
         if not isinstance(path, yarl.URL):
             path = yarl.URL(str(path))
         data = loader.load(Plugins(plugins or []), path)
-        return cls.loads(url, data, session_factory, loader, plugins)
+        return cls.loads(url, data, session_factory, loader, plugins, use_operation_tags)
 
     @classmethod
     def loads(
@@ -97,11 +103,12 @@ class OpenAPI:
         session_factory: Callable[[], httpx.AsyncClient] = httpx.AsyncClient,
         loader=None,
         plugins: List[Plugin] = None,
+        use_operation_tags: bool = False,
     ):
         if loader is None:
             loader = NullLoader()
         data = loader.parse(Plugins(plugins or []), yarl.URL(url), data)
-        return cls(url, data, session_factory, loader, plugins)
+        return cls(url, data, session_factory, loader, plugins, use_operation_tags)
 
     def _parse_obj(self, raw_document):
         v = raw_document.get("openapi", None)
@@ -135,6 +142,7 @@ class OpenAPI:
         session_factory: Callable[[], Union[httpx.Client, httpx.AsyncClient]] = httpx.AsyncClient,
         loader=None,
         plugins: List[Plugin] = None,
+        use_operation_tags: bool = True,
     ):
         """
         Creates a new OpenAPI document from a loaded spec file.  This is
@@ -145,6 +153,9 @@ class OpenAPI:
         :type document: dct
         :param session_factory: default uses new session for each call, supply your own if required otherwise.
         :type session_factory: returns httpx.AsyncClient or http.Client
+        :param use_operation_tags: honor tags
+        :type use_operation_tags: bool
+
         """
 
         self._base_url: yarl.URL = yarl.URL(url)
@@ -170,12 +181,12 @@ class OpenAPI:
         """
         the related documents
         """
-        self._documents: Dict[str, RootBase] = dict()
+        self._documents: Dict[yarl.URL, RootBase] = dict()
 
         """
         the plugin interface allows taking care of defects in description documents and implementations
         """
-        self.plugins: Plugins = Plugins(plugins or [])
+        self._init_plugins(plugins)
 
         log.init()
         self.log = logging.getLogger("aiopenapi3.OpenAPI")
@@ -184,20 +195,18 @@ class OpenAPI:
 
         self._root = self._parse_obj(document)
 
-        self._init_plugins(plugins)
-
-        self._documents[pathlib.Path(pathlib.Path(yarl.URL(url).path).parts[-1])] = self._root
+        self._documents[yarl.URL(url)] = self._root
 
         self._init_session_factory(session_factory)
         self._init_references()
-        self._init_operationindex()
+        self._init_operationindex(use_operation_tags)
         self._init_schema_types()
 
         self.plugins.init.initialized(initialized=self._root)
 
     def _init_plugins(self, plugins):
         for i in plugins or []:
-            i.root = self._root
+            i.api = self
         self.plugins = Plugins(plugins or [])
 
     def _init_session_factory(self, session_factory):
@@ -238,27 +247,23 @@ class OpenAPI:
     #        for i in self._documents.values():
     #            i._resolve_references(self)
 
-    def _init_operationindex(self):
-        operation_map = set()
-
-        def test_operation(operation_id):
-            if operation_id in operation_map:
-                raise SpecError(f"Duplicate operationId {operation_id}", element=None)
-            operation_map.add(operation_id)
-
+    def _init_operationindex(self, use_operation_tags: bool):
         if isinstance(self._root, v20.Root):
             if self.paths:
                 for path, obj in self.paths.items():
                     for m in obj.__fields_set__ & HTTP_METHODS:
                         op = getattr(obj, m)
-                        _validate_parameters(op, path)
+                        op._validate_path_parameters(obj, path)
                         if op.operationId is None:
                             continue
-                        formatted_operation_id = op.operationId.replace(" ", "_")
-                        test_operation(formatted_operation_id)
                         for r, response in op.responses.items():
                             if isinstance(response, Reference):
                                 continue
+                            if response.headers:
+                                for h in response.headers.values():
+                                    items = v20.Schema(type=h.items.type) if h.items else None
+                                    h._schema = v20.Schema(type=h.type, items=items)
+
                             if isinstance(response.schema_, (v20.Schema,)):
                                 response.schema_._get_identity("OP", f"{path}.{m}.{r}")
 
@@ -274,13 +279,10 @@ class OpenAPI:
                 for path, obj in self.paths.items():
                     for m in obj.__fields_set__ & HTTP_METHODS:
                         op = getattr(obj, m)
-                        _validate_parameters(op, path)
+                        op._validate_path_parameters(obj, path)
                         if op.operationId is None:
                             continue
-                        formatted_operation_id = op.operationId.replace(" ", "_")
-                        test_operation(formatted_operation_id)
                         for r, response in op.responses.items():
-
                             if isinstance(response, Reference):
                                 continue
                             for c, content in response.content.items():
@@ -288,10 +290,14 @@ class OpenAPI:
                                     continue
                                 if isinstance(content.schema_, (v30.Schema, v31.Schema)):
                                     content.schema_._get_identity("OP", f"{path}.{m}.{r}.{c}")
+            else:
+                self._root.paths = dict()
 
         else:
             raise ValueError(self._root)
 
+        self._operationindex = OperationIndex(self, use_operation_tags)
+
     @staticmethod
     def _iterate_schemas(schemas, d, r):
         if not d:
@@ -406,7 +412,10 @@ class OpenAPI:
         for name, schema in types.items():
             if not (inspect.isclass(schema) and issubclass(schema, BaseModel)):
                 continue
-            schema.update_forward_refs(**types)
+            try:
+                schema.update_forward_refs(**types)
+            except Exception as e:
+                raise e
 
     @property
     def url(self):
@@ -466,9 +475,9 @@ class OpenAPI:
 
     @property
     def _(self):
-        return OperationIndex(self)
+        return self._operationindex
 
-    def resolve_jr(self, root: "Rootv30", obj, value: Reference):
+    def resolve_jr(self, root: RootBase, obj, value: Reference):
         url, jp = JSONReference.split(value.ref)
         if url != "":
             url = yarl.URL(url)
@@ -483,17 +492,3 @@ class OpenAPI:
             # add metadata to the error
             e.element = obj
             raise
-
-
-def _validate_parameters(op: "Operation", path):
-    """
-    Ensures that all parameters for this path are valid
-    """
-    assert isinstance(path, str)
-    allowed_path_parameters = frozenset(re.findall(r"{([a-zA-Z0-9\-\._~]+)}", path))
-
-    path_parameters = frozenset(map(lambda x: x.name, filter(lambda c: c.in_ == "path", op.parameters)))
-
-    r = path_parameters - allowed_path_parameters
-    if r:
-        raise SpecError(f"Parameter name{'s' if len(r) > 1 else ''} not found in path: {', '.join(sorted(r))}")
