@@ -1,6 +1,5 @@
-from typing import Dict, List
+from typing import List, Union, cast
 import json
-import io
 
 import httpx
 import pydantic
@@ -28,7 +27,7 @@ class Request(RequestBase):
         return self._data_parameter.schema_
 
     @property
-    def parameters(self) -> Dict[str, ParameterBase]:
+    def parameters(self) -> List[ParameterBase]:
         return list(
             filter(lambda x: x.in_ != "body", self.operation.parameters + self.root.paths[self.path].parameters)
         )
@@ -72,15 +71,17 @@ class Request(RequestBase):
                 f"No security requirement satisfied (accepts {options} given {{{' and '.join(sorted(self.security.keys()))}}})"
             )
 
-    def _prepare_secschemes(self, scheme: str, value: List[str]):
+    def _prepare_secschemes(self, scheme: str, value: Union[str, List[str]]):
         """
         https://swagger.io/specification/v2/#security-scheme-object
         """
         ss = self.root.securityDefinitions[scheme]
 
         if ss.type == "basic":
+            value = cast(List[str], value)
             self.req.auth = httpx.BasicAuth(*value)
 
+        value = cast(str, value)
         if ss.type == "apiKey":
             if ss.in_ == "query":
                 # apiKey in query parameter
@@ -90,45 +91,52 @@ class Request(RequestBase):
                 # apiKey in query header data
                 self.req.headers[ss.name] = value
 
-    def _prepare_parameters(self, parameters):
-        parameters = parameters or dict()
-        accepted_parameters = {_.name: _ for _ in self.operation.parameters + self.root.paths[self.path].parameters}
+    def _prepare_parameters(self, provided):
+        provided = provided or dict()
+        possible = {_.name: _ for _ in self.operation.parameters + self.root.paths[self.path].parameters}
 
-        provided = frozenset(parameters.keys())
-        accepted = frozenset(accepted_parameters.keys())
+        parameters = {i.name: i.default for i in filter(lambda x: x.default is not None, possible.values())}
+        parameters.update(provided)
+
+        available = frozenset(parameters.keys())
+        accepted = frozenset(possible.keys())
         required = frozenset(
-            map(lambda x: x[0], filter(lambda y: y[1].required and y[1].in_ != "body", accepted_parameters.items()))
+            map(lambda x: x[0], filter(lambda y: y[1].required and y[1].in_ != "body", possible.items()))
         )
-        if provided > accepted:
-            raise ValueError(f"Parameter {sorted(provided - accepted)} unknown (accepted {sorted(accepted)})")
-        if required > provided:
-            raise ValueError(f"Required Parameter {sorted(required - provided)} missing (provided {sorted(provided)})")
+        if available - accepted:
+            raise ValueError(f"Parameter {sorted(available - accepted)} unknown (accepted {sorted(accepted)})")
+        if required - available:
+            raise ValueError(
+                f"Required Parameter {sorted(required - available)} missing (provided {sorted(available)})"
+            )
 
         path_parameters = {}
+
         for name, value in parameters.items():
-            spec = accepted_parameters[name]
+            spec = possible[name]
+
+            values = spec._encode(name, value)
+            assert isinstance(values, dict)
 
             if spec.in_ == "formData":
                 if "multipart/form-data" not in self.operation.consumes:
                     raise ValueError(f"operation does not consume form data but parameter {name} is formData")
                 if spec.type == "file":
-                    # https://www.python-httpx.org/quickstart/#sending-multipart-file-uploads
-                    assert type(value) == tuple and len(value) == 3 and isinstance(value[1], io.IOBase)
-                    self.req.files[name] = value
+                    self.req.files.update(values)
                 else:
-                    self.req.data[name] = value
+                    self.req.data.update(values)
 
             if spec.in_ == "path":
                 # The string method `format` is incapable of partial updates,
                 # as such we need to collect all the path parameters before
                 # applying them to the format string.
-                path_parameters[name] = value
+                path_parameters.update(values)
 
             if spec.in_ == "query":
-                self.req.params[name] = value
+                self.req.params.update(values)
 
             if spec.in_ == "header":
-                self.req.headers[name] = value
+                self.req.headers.update(values)
 
         self.req.url = self.req.url.format(**path_parameters)
 
@@ -179,6 +187,7 @@ class Request(RequestBase):
         return req
 
     def _process(self, result):
+        headers = dict()
         # spec enforces these are strings
         status_code = str(result.status_code)
 
@@ -198,8 +207,23 @@ class Request(RequestBase):
                 result,
             )
 
+        if expected_response.headers:
+            # FIXME
+            # there is no "required" field - but it is referenced.
+            # https://github.com/OAI/OpenAPI-Specification/blob/main/versions/2.0.md#header-object
+            # required = frozenset(map(lambda x: x[0].lower(), filter(lambda x: x[1].required is True, expected_response.headers.items())))
+            #
+            # required = frozenset()
+            # available = frozenset(result.headers.keys())
+            # if required - available:
+            #     raise ValueError(f"missing {sorted(required - available)}")
+            for name, header in expected_response.headers.items():
+                data = result.headers.get(name, None)
+                if data:
+                    headers[name] = header._schema.model(header._decode(data))
+
         if status_code == "204":
-            return
+            return headers, None
 
         content_type = result.headers.get("Content-Type", None)
 
@@ -217,9 +241,9 @@ class Request(RequestBase):
             data = self.api.plugins.message.unmarshalled(
                 operationId=self.operation.operationId, unmarshalled=data
             ).unmarshalled
-            return data
+            return headers, data
         elif content_type in self.operation.produces:
-            return result.content
+            return headers, result.content
         else:
             raise ContentTypeError(
                 content_type,
