@@ -1,4 +1,4 @@
-from typing import Dict, List
+from typing import List, Union, cast
 import json
 
 import httpx
@@ -28,7 +28,7 @@ class Request(RequestBase):
         return self.operation.requestBody.content["application/json"].schema_
 
     @property
-    def parameters(self) -> Dict[str, ParameterBase]:
+    def parameters(self) -> List[ParameterBase]:
         return self.operation.parameters + self.root.paths[self.path].parameters
 
     def args(self, content_type: str = "application/json"):
@@ -70,7 +70,7 @@ class Request(RequestBase):
                 f"No security requirement satisfied (accepts {options} given {{{' and '.join(sorted(self.security.keys()))}}}"
             )
 
-    def _prepare_secschemes(self, scheme: str, value: List[str]):
+    def _prepare_secschemes(self, scheme: str, value: Union[str, List[str]]):
         ss = self.root.components.securitySchemes[scheme]
 
         if ss.type == "http" and ss.scheme_ == "basic":
@@ -79,6 +79,7 @@ class Request(RequestBase):
         if ss.type == "http" and ss.scheme_ == "digest":
             self.req.auth = httpx.DigestAuth(*value)
 
+        value = cast(str, value)
         if ss.type == "http" and ss.scheme_ == "bearer":
             header = ss.bearerFormat or "Bearer {}"
             self.req.headers["Authorization"] = header.format(value)
@@ -99,36 +100,52 @@ class Request(RequestBase):
             if ss.in_ == "cookie":
                 self.req.cookies = {ss.name: value}
 
-    def _prepare_parameters(self, parameters):
-        parameters = parameters or dict()
-        accepted_parameters = {_.name: _ for _ in self.operation.parameters + self.root.paths[self.path].parameters}
+    def _prepare_parameters(self, provided):
+        """
+        assigns the parameters provided to the header/path/cookie …
 
-        provided = frozenset(parameters.keys())
-        accepted = frozenset(accepted_parameters.keys())
-        required = frozenset(map(lambda x: x[0], filter(lambda y: y[1].required, accepted_parameters.items())))
-        if provided > accepted:
-            raise ValueError(f"Parameter {sorted(provided - accepted)} unknown (accepted {sorted(accepted)})")
-        if required > provided:
-            raise ValueError(f"Required Parameter {sorted(required - provided)} missing (provided {sorted(provided)})")
+        FIXME: handle parameter location
+          https://spec.openapis.org/oas/v3.0.3#parameter-object
+          A unique parameter is defined by a combination of a name and location.
+        """
+        provided = provided or dict()
+        possible = {_.name: _ for _ in self.operation.parameters + self.root.paths[self.path].parameters}
+
+        parameters = {
+            i.name: i.schema_.default for i in filter(lambda x: x.schema_.default is not None, possible.values())
+        }
+        parameters.update(provided)
+
+        available = frozenset(parameters.keys())
+        accepted = frozenset(possible.keys())
+        required = frozenset(map(lambda x: x[0], filter(lambda y: y[1].required, possible.items())))
+        if available - accepted:
+            raise ValueError(f"Parameter {sorted(available - accepted)} unknown (accepted {sorted(accepted)})")
+        if required - available:
+            raise ValueError(
+                f"Required Parameter {sorted(required - available)} missing (provided {sorted(available)})"
+            )
 
         path_parameters = {}
-        for name, value in parameters.items():
-            spec = accepted_parameters[name]
 
+        for name, value in parameters.items():
+            spec = possible[name]
+            values = spec._encode(name, value)
+            assert isinstance(values, dict)
             if spec.in_ == "path":
                 # The string method `format` is incapable of partial updates,
                 # as such we need to collect all the path parameters before
                 # applying them to the format string.
-                path_parameters[name] = value
+                path_parameters.update(values)
 
             if spec.in_ == "query":
-                self.req.params[name] = value
+                self.req.params.update(values)
 
             if spec.in_ == "header":
-                self.req.headers[name] = value
+                self.req.headers.update(values)
 
             if spec.in_ == "cookie":
-                self.req.cookies[name] = value
+                self.req.cookies.update(values)
 
         self.req.url = self.req.url.format(**path_parameters)
 
@@ -186,6 +203,7 @@ class Request(RequestBase):
         return req
 
     def _process(self, result):
+        headers = dict()
         # spec enforces these are strings
         status_code = str(result.status_code)
 
@@ -205,8 +223,20 @@ class Request(RequestBase):
                 result,
             )
 
+        if expected_response.headers:
+            required = frozenset(
+                map(lambda x: x[0].lower(), filter(lambda x: x[1].required is True, expected_response.headers.items()))
+            )
+            available = frozenset(result.headers.keys())
+            if required - available:
+                raise ValueError(f"missing Header {sorted(required - available)}")
+            for name, header in expected_response.headers.items():
+                data = result.headers.get(name, None)
+                if data:
+                    headers[name] = header.schema_.model(header._decode(data))
+
         if len(expected_response.content) == 0:
-            return None
+            return headers, None
 
         content_type = result.headers.get("Content-Type", None)
 
@@ -246,7 +276,7 @@ class Request(RequestBase):
             data = self.api.plugins.message.unmarshalled(
                 operationId=self.operation.operationId, unmarshalled=data
             ).unmarshalled
-            return data
+            return headers, data
         else:
             raise NotImplementedError()
 
